/*
 * Copyright (c) 2014 Nordic Semiconductor. All Rights Reserved.
 *
 * The information contained herein is confidential property of Nordic Semiconductor. The use,
 * copying, transfer or disclosure of such information is prohibited except by express written
 * agreement with Nordic Semiconductor.
 *
 */

#include <stdint.h>
#include <string.h>
#include "nrf.h"
#include "nordic_common.h"
#include "ble.h"
#include "ble_gap.h"
#include "app_error.h"
#include "app_timer.h"
#include "app_gpiote.h"
#include "app_button.h"
#include "ble_advdata.h"
#include "boards.h"
#include "nrf_gpio.h"
#include "nrf_sdm.h"
#include "softdevice_handler.h"
#include "pstorage.h"
#include "device_manager.h"
#include "bsp.h"

//a UART SPI
#include "app_uart.h"
#include "spi_master.h"
#include "nrf_delay.h"

// SPI_MASTER_0_ENABLE unter Targetoptionen -> C -> Define eintragen
//b

#define IS_SRVC_CHANGED_CHARACT_PRESENT    0                                          /**< Include or not the service_changed characteristic. If not enabled, the server's database cannot be changed for the lifetime of the device. */

#define SEND_NOTIFICATION_BUTTON_ID        0                                          /**< Id for button used for sending notification button. */
#define BOND_DELETE_ALL_BUTTON_ID          1                                          /**< Id for button used for deleting bonds on start button. */

#define DEVICE_NAME                        "Multilink"                                /**< Name of device. Will be included in the advertising data. */

#define APP_TIMER_PRESCALER                0                                          /**< Value of the RTC1 PRESCALER register. */
#define APP_TIMER_MAX_TIMERS               (1 + BSP_APP_TIMERS_NUMBER + 1) //c +1               /**< Maximum number of simultaneously created timers. */
#define APP_TIMER_OP_QUEUE_SIZE            4                                          /**< Size of timer operation queues. */

#define APP_GPIOTE_MAX_USERS               2                                          /**< Maximum number of users of the GPIOTE handler. */

#define BUTTON_DETECTION_DELAY             APP_TIMER_TICKS(50, APP_TIMER_PRESCALER)   /**< Delay from a GPIOTE event until a button is reported as pushed (in number of timer ticks). */

#define APP_ADV_INTERVAL                   MSEC_TO_UNITS(50, UNIT_0_625_MS)           /**< The advertising interval (in units of 0.625 ms. This value corresponds to 40 ms). */
#define APP_ADV_TIMEOUT_IN_SECONDS         0 //c kein Timeout                         /**< The advertising timeout (in units of seconds). */

#define MIN_CONN_INTERVAL                  MSEC_TO_UNITS(2000, UNIT_1_25_MS)          /**< Minimum acceptable connection interval (0.5 seconds). */
#define MAX_CONN_INTERVAL                  MSEC_TO_UNITS(4000, UNIT_1_25_MS)          /**< Maximum acceptable connection interval (1 second). */
#define SLAVE_LATENCY                      0                                          /**< Slave latency. */
#define CONN_SUP_TIMEOUT                   MSEC_TO_UNITS(8100, UNIT_10_MS)            /**< Connection supervisory timeout (4 seconds). */

#define MULTILINK_PERIPHERAL_BASE_UUID     {{0xB3, 0x58, 0x55, 0x40, 0x50, 0x60, 0x11, \
                                           0xe3, 0x8f, 0x96, 0x08, 0x00, 0x00, 0x00,   \
                                           0x9a, 0x66}}                               /**< 128bit UUID base used for example. */
#define MULTILINK_PERIPHERAL_SERVICE_UUID  0x9001                                     /**< Serrvice UUID over the 128-bit base used for the example. */
#define MULTILINK_PERIPHERAL_CHAR_UUID     0x900A                                     /**< Characteristic UUID over the 128-bit base used for the example. */

#define SEC_PARAM_TIMEOUT                  30                                         /**< Timeout for Pairing Request or Security Request (in seconds). */
#define SEC_PARAM_BOND                     1                                          /**< Perform bonding. */
#define SEC_PARAM_MITM                     0                                          /**< Man In The Middle protection not required. */
#define SEC_PARAM_IO_CAPABILITIES          BLE_GAP_IO_CAPS_NONE                       /**< No I/O capabilities. */
#define SEC_PARAM_OOB                      0                                          /**< Out Of Band data not available. */
#define SEC_PARAM_MIN_KEY_SIZE             7                                          /**< Minimum encryption key size. */
#define SEC_PARAM_MAX_KEY_SIZE             16


static uint16_t                  m_conn_handle = BLE_CONN_HANDLE_INVALID;             /**< Connection handle. */
static ble_gatts_char_handles_t  m_char_handles;                                      /**< GATT characteristic definition handles. */
static uint8_t                   m_base_uuid_type;                                    /**< UUID type. */
static dm_application_instance_t m_app_handle;                                        /**< Application instance allocated by device manager to the application. */

static bool                      m_memory_access_in_progress = false;                 /**< Flag to keep track of ongoing operations on persistent memory. */


/**@brief Function for asserts in the SoftDevice.
 *
 * @details This function will be called in case of an assert in the SoftDevice.
 *
 * @warning This handler is an example only and does not fit a final product. You need to analyze
 *          how your product is supposed to react in case of Assert.
 * @warning On assert from the SoftDevice, the system can only recover on reset.
 *
 * @param[in] line_num    Line number of the failing ASSERT call.
 * @param[in] p_file_name File name of the failing ASSERT call.
 */
void assert_nrf_callback(uint16_t line_num, const uint8_t * p_file_name)
{
    app_error_handler(0xDEADBEEF, line_num, p_file_name);
}
//a ADC Spannung
		uint8_t messe_spannung(void) {
		NRF_ADC->CONFIG     = (ADC_CONFIG_RES_8bit                        << ADC_CONFIG_RES_Pos)     |
                          (ADC_CONFIG_INPSEL_SupplyOneThirdPrescaling << ADC_CONFIG_INPSEL_Pos)  |
                          (ADC_CONFIG_REFSEL_VBG                      << ADC_CONFIG_REFSEL_Pos)  |
                          (ADC_CONFIG_PSEL_Disabled                   << ADC_CONFIG_PSEL_Pos)    |
                          (ADC_CONFIG_EXTREFSEL_None                  << ADC_CONFIG_EXTREFSEL_Pos);
						NRF_ADC->ENABLE = ADC_ENABLE_ENABLE_Enabled;							
						NRF_ADC->TASKS_START = 1;
						nrf_delay_ms(1);
						NRF_ADC->ENABLE = ADC_ENABLE_ENABLE_Disabled;
						return NRF_ADC->RESULT;
					}
//b
//a SPI

static uint8_t m_tx_data_1B[8u] = {0}; /**< A buffer with data to transfer. */
static uint8_t m_rx_data_1B[8u] = {0}; /**< A buffer for incoming data. */
//static uint8_t m_tx_data_2B[16u] = {0}; /**< A buffer with data to transfer. */
//static uint8_t m_rx_data_2B[16u] = {0}; /**< A buffer for incoming data. */
static uint8_t m_tx_data_3B[24u] = {0}; /**< A buffer with data to transfer. */
static uint8_t m_rx_data_3B[24u] = {0}; /**< A buffer for incoming data. */
static uint8_t m_tx_data_4B[32u] = {0}; /**< A buffer with data to transfer. */
static uint8_t m_rx_data_4B[32u] = {0}; /**< A buffer for incoming data. */
static volatile bool m_transfer_completed = true; /**< A flag to inform about completed transfer. */

static void spi_master_event_handler(spi_master_evt_t spi_master_evt)
{
    uint32_t err_code = NRF_SUCCESS;

    switch (spi_master_evt.evt_type)
    {
        case SPI_MASTER_EVT_TRANSFER_COMPLETED:
	

            err_code = bsp_indication_set(BSP_INDICATE_RCV_OK);
            APP_ERROR_CHECK(err_code);

            // Inform application that transfer is completed.
            m_transfer_completed = true;
            break;

        default:
            // No implementation needed.
            break;
    }
}

static uint32_t spi_master_init(void)
{
		uint32_t err_code = NRF_SUCCESS;
    spi_master_config_t spi_config = SPI_MASTER_INIT_DEFAULT; 
		spi_config.SPI_CONFIG_CPOL = SPI_CONFIG_CPOL_ActiveLow; 	// beides 
		spi_config.SPI_CONFIG_CPHA = SPI_CONFIG_CPHA_Trailing;		// oder beides nicht
    spi_config.SPI_CONFIG_ORDER = SPI_CONFIG_ORDER_MsbFirst;
		spi_config.SPI_Freq = SPI_FREQUENCY_FREQUENCY_M4;
    spi_config.SPI_Pin_SCK  = 16;
    spi_config.SPI_Pin_MISO = 14;
    spi_config.SPI_Pin_MOSI = 13;
    spi_config.SPI_Pin_SS   = 12;
		err_code = spi_master_open(SPI_MASTER_0, &spi_config);
    APP_ERROR_CHECK(err_code);
    
		return spi_master_open(SPI_MASTER_0, &spi_config);
}

//b

//a Variablen fuer Druck und Temperatur
static uint16_t Koeff[8] = {0};
static uint32_t D1 = 0;
static uint32_t D2 = 0; 
static int32_t dT = 0;
static int32_t T = 0;
static int64_t Offset = 0;
static int64_t Sensitivity  = 0;
static int64_t T2 = 0;
static int64_t OFF = 0;
static int64_t Sens = 0;
static int32_t P = 0;
static uint8_t p_crc,n_crc = 0;
//b

//a
void sensor_reset(void)
{
			if (m_transfer_completed) 
			{
			m_transfer_completed = false;	
			m_tx_data_1B[0] = 0x1E; 
			m_rx_data_1B[0] = 0;
			//printf("%s \n","Sende Reset");
			uint32_t err_code = spi_master_send_recv(SPI_MASTER_0, m_tx_data_1B, 1, m_rx_data_1B, 1); 					
			APP_ERROR_CHECK(err_code);
			//printf("%X \n",m_rx_data_1B[0]);
			nrf_delay_ms(10);
			}
}

void hole_koeffizienten(void)
{
					for (int i = 0; i < 8; i++ ){
					//printf("%X: \n",i);  
					if (m_transfer_completed)	
				{
					uint8_t Koeffnr = 0xA0+(i*2);
         //Command for Read PROM + PROM Read
          m_transfer_completed = false;					
					m_tx_data_3B[0] = Koeffnr;
					m_tx_data_3B[1] = 0xFF;
					m_tx_data_3B[2] = 0xFF;
					uint32_t err_code = spi_master_send_recv(SPI_MASTER_0, m_tx_data_3B, 3, m_rx_data_3B, 3);	
					APP_ERROR_CHECK(err_code);					
					Koeff[i] = m_rx_data_3B[1] << 8 | m_rx_data_3B[2];
				}}
}

bool crc_check(void) {
	
	

		p_crc = Koeff[7];
    uint16_t n_rem;		// crc reminder
    uint16_t crc_read;	// gelesener CRC
    uint8_t  n_bit;
    n_rem = 0x00;
    crc_read = Koeff[7];
	
    Koeff[7] = (0xFF00 & (Koeff[7])); // CRC byte replaced with 0
    for (int cnt = 0; cnt < 16; cnt++)
    {
        if (cnt%2 == 1) {
        	n_rem ^= (unsigned short)((Koeff[cnt>>1]) & 0x00FF);
        }
        else {
        	n_rem ^= (unsigned short)(Koeff[cnt>>1] >> 8);
        }
        for (n_bit = 8; n_bit > 0; n_bit--)
        {
            if (n_rem & (0x8000))
            {
                n_rem = (n_rem << 1) ^ 0x3000;
            }
            else {
                n_rem = (n_rem << 1);
            }
        }
    }
    n_rem = (0x000F & (n_rem >> 12));// // final 4-bit reminder is CRC code
    Koeff[7] = crc_read; 
    n_crc = n_rem ^ 0x00; 

	
	if (p_crc != n_crc) {
        return false;
    } 
    return true;
}

static int32_t hole_temperatur(void)
{
				
					if (m_transfer_completed) 
				{	// ADC Temperatur 256 Aufloesung
					m_transfer_completed = false;	
					m_tx_data_1B[0] = 0x50; 
					uint32_t err_code = spi_master_send_recv(SPI_MASTER_0, m_tx_data_1B, 1, m_rx_data_1B, 1);
					nrf_delay_ms(1);
				}
				
					if (m_transfer_completed)
        { //Hole und berechne Temperatur
          m_transfer_completed = false;					
					m_tx_data_4B[0] = 0x00;
					m_tx_data_4B[1] = 0xFF;
					m_tx_data_4B[2] = 0xFF;
					m_tx_data_4B[3] = 0xFF;
					uint32_t err_code = spi_master_send_recv(SPI_MASTER_0, m_tx_data_4B, 4, m_rx_data_4B, 4);	
					APP_ERROR_CHECK(err_code);					
					nrf_delay_ms(1);
					//printf("%X \n",m_rx_data_4B[1]);
					//printf("%X \n",m_rx_data_4B[2]);
					//printf("%X \n",m_rx_data_4B[3]);
					D2 = m_rx_data_4B[1]<<16 | m_rx_data_4B[2]<<8 | m_rx_data_4B[3];
					//printf("%d \n",D2);
					//printf("%s \n", m_transfer_completed ? "true" : "false");
					dT = (int32_t)D2 - ( (int32_t)Koeff[5] * 256 );				
					T = 2000 + ((int64_t)dT * Koeff[6]) / 8388608LL;					
				} return T = (int32_t)T;
}


static uint32_t hole_druck(void)
{
					// ADC Druck 4096 Aufloesung
					if (m_transfer_completed) 
				{
					m_transfer_completed = false;	
					m_tx_data_1B[0] = 0x48; 
					uint32_t err_code = spi_master_send_recv(SPI_MASTER_0, m_tx_data_1B, 1, m_rx_data_1B, 1);
					nrf_delay_ms(10);
				}
				
					if (m_transfer_completed)
        { //Hole und berechne Druck
          m_transfer_completed = false;					
					m_tx_data_4B[0] = 0x00;
					m_tx_data_4B[1] = 0xFF;
					m_tx_data_4B[2] = 0xFF;
					m_tx_data_4B[3] = 0xFF;
					uint32_t err_code = spi_master_send_recv(SPI_MASTER_0, m_tx_data_4B, 4, m_rx_data_4B, 4);	
					APP_ERROR_CHECK(err_code);					
					nrf_delay_ms(1);
					D1 = m_rx_data_4B[1]<<16 | m_rx_data_4B[2]<<8 | m_rx_data_4B[3];
						if (T < 2000) {
							// Temp unter 20.0C
								T2 = 3 * ((int64_t)dT * dT)  / 8589934592LL ;
								T2 = (int32_t)T2;
								OFF = 3 * ((T-2000) * (T-2000)) / 8 ;
								Sens = 7 * ((T-2000) * (T-2000)) / 8 ;
							} else { // T > (20.0C)
								T2 = 0;
								OFF = 0;
								Sens = 0;
						}
					Offset = (int64_t)Koeff[2] * 262144 + Koeff[4] * (int64_t)dT / 32;
					Sensitivity = (int64_t)Koeff[1] * 131072 + Koeff[3] * (int64_t)dT / 128;
					T = T - T2;
					Offset = Offset - OFF;
					Sensitivity = Sensitivity - Sens;
					P = ((D1 * Sensitivity) / 2097152 - Offset) / 32768;					
				}	return P;
}

void messwerte_dezimal_per_uart(uint16_t messintervall)
{
		float T_float  = (float)hole_temperatur() / 100;
		float P_float = (float)hole_druck() / 100;
		printf("Messwerte: %4.2fC - %6.2fmbar \n", T_float,P_float);
		if (messintervall>14)	nrf_delay_ms(messintervall-14);
}

void messreihe(uint16_t messintervall)
{
	printf("%d;%d \n",hole_temperatur(),hole_druck());
	if (messintervall>14)	nrf_delay_ms(messintervall-14);
}

//b


/**@brief Function for the GAP initialization.
 *
 * @details This function sets up all the necessary GAP (Generic Access Profile) parameters of the
 *          device including the device name, appearance, and the preferred connection parameters.
 */
static void gap_params_init(void)
{
    uint32_t                err_code;
    ble_gap_conn_params_t   gap_conn_params;
    ble_gap_conn_sec_mode_t sec_mode;

    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&sec_mode);

    err_code = sd_ble_gap_device_name_set(&sec_mode,
                                          (const uint8_t *)DEVICE_NAME,
                                          strlen(DEVICE_NAME));
    APP_ERROR_CHECK(err_code);

    memset(&gap_conn_params, 0, sizeof(gap_conn_params));

    gap_conn_params.min_conn_interval = MIN_CONN_INTERVAL;
    gap_conn_params.max_conn_interval = MAX_CONN_INTERVAL;
    gap_conn_params.slave_latency     = SLAVE_LATENCY;
    gap_conn_params.conn_sup_timeout  = CONN_SUP_TIMEOUT;

    err_code = sd_ble_gap_ppcp_set(&gap_conn_params);
    APP_ERROR_CHECK(err_code);
}


/**@brief Function for initializing the Advertising functionality.
 *
 * @details Encodes the required advertising data and passes it to the stack.
 *          Also builds a structure to be passed to the stack when starting advertising.
 */
static void advertising_init(void)
{
    uint32_t      err_code;
    ble_advdata_t advdata;
    uint8_t       flags = BLE_GAP_ADV_FLAGS_LE_ONLY_GENERAL_DISC_MODE; //c General Mode (Advertised ohne Zeitlimit bis CONN

    // Build and set advertising data.
    memset(&advdata, 0, sizeof(advdata));

    advdata.name_type               = BLE_ADVDATA_FULL_NAME;
    advdata.include_appearance      = false;
    advdata.flags                   = flags;
    advdata.uuids_complete.uuid_cnt = 0;

    err_code = ble_advdata_set(&advdata, NULL);
    APP_ERROR_CHECK(err_code);
}


/**@brief Function for starting advertising.
 */
static void advertising_start(void)
{
    uint32_t             err_code;
    ble_gap_adv_params_t adv_params;
    uint32_t             count;

    // Verify if there is any flash access pending, if yes delay starting advertising until
    // it's complete.
    err_code = pstorage_access_status_get(&count);
    APP_ERROR_CHECK(err_code);

    if (count != 0)
    {
        m_memory_access_in_progress = true;
        return;
    }

    // Start advertising
    memset(&adv_params, 0, sizeof(adv_params));

    adv_params.type        = BLE_GAP_ADV_TYPE_ADV_IND;
    adv_params.p_peer_addr = NULL;
    adv_params.fp          = BLE_GAP_ADV_FP_ANY;
    adv_params.interval    = APP_ADV_INTERVAL;
    adv_params.timeout     = APP_ADV_TIMEOUT_IN_SECONDS;

    err_code = sd_ble_gap_adv_start(&adv_params);
    APP_ERROR_CHECK(err_code);

    err_code = bsp_indication_set(BSP_INDICATE_ADVERTISING);
    APP_ERROR_CHECK(err_code);
}


/**@brief Function for initializing services that will be used by the application.
 *
 * @details Initialize the multilink custom service.
 */
static void services_init(void)
{
    uint32_t            err_code;
    ble_uuid_t          uuid;
    ble_gatts_char_md_t char_md;
    ble_gatts_attr_t    attr;
    ble_gatts_attr_md_t attr_md;
    ble_gatts_attr_md_t cccd_md;
    ble_gatts_attr_md_t char_ud_md;
    uint16_t            svc_test;

    static uint8_t multilink_peripheral_data;
		static uint8_t multilink_peripheral_ud[] = "Noch keine Messwerte vorhanden"; //c

    ble_uuid128_t base_uuid = MULTILINK_PERIPHERAL_BASE_UUID;

    err_code = sd_ble_uuid_vs_add(&base_uuid, &m_base_uuid_type);
    APP_ERROR_CHECK(err_code);

    uuid.type = m_base_uuid_type;
    uuid.uuid = MULTILINK_PERIPHERAL_SERVICE_UUID;

    err_code = sd_ble_gatts_service_add(BLE_GATTS_SRVC_TYPE_PRIMARY, &uuid, &svc_test);
    APP_ERROR_CHECK(err_code);

    uuid.uuid = MULTILINK_PERIPHERAL_CHAR_UUID;

    memset(&attr, 0, sizeof(ble_gatts_attr_t));
    attr.p_uuid    = &uuid;
    attr.p_attr_md = &attr_md;
    attr.max_len   = 3; //c 1->2
    attr.p_value   = &multilink_peripheral_data;
    attr.init_len  = sizeof(multilink_peripheral_data);

    memset(&attr_md, 0, sizeof(ble_gatts_attr_md_t));
    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&attr_md.read_perm);
    BLE_GAP_CONN_SEC_MODE_SET_ENC_NO_MITM(&attr_md.write_perm);
    attr_md.vloc = BLE_GATTS_VLOC_STACK;
    attr_md.vlen = 0;

    memset(&cccd_md, 0, sizeof(ble_gatts_attr_md_t));
    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&cccd_md.read_perm);
    BLE_GAP_CONN_SEC_MODE_SET_ENC_NO_MITM(&cccd_md.write_perm);
    cccd_md.vloc = BLE_GATTS_VLOC_STACK;

    memset(&char_md, 0, sizeof(ble_gatts_char_md_t));
    char_md.p_cccd_md               = &cccd_md;
    char_md.char_props.notify       = 1;
    char_md.char_props.indicate     = 1;
    char_md.char_props.read         = 1;
    char_md.char_props.write        = 1;
    char_md.char_ext_props.wr_aux   = 1;
    char_md.p_user_desc_md          = &char_ud_md;
    char_md.p_char_user_desc        = multilink_peripheral_ud;
    char_md.char_user_desc_size     = (uint8_t)strlen((char *)multilink_peripheral_ud);
    char_md.char_user_desc_max_size = (uint8_t)strlen((char *)multilink_peripheral_ud);

    memset(&char_ud_md, 0, sizeof(ble_gatts_attr_md_t));
    char_ud_md.vloc = BLE_GATTS_VLOC_STACK;
    char_ud_md.vlen = 1;
    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&char_ud_md.read_perm);
    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&char_ud_md.write_perm);

    err_code = sd_ble_gatts_characteristic_add(BLE_GATT_HANDLE_INVALID,
                                               &char_md,
                                               &attr,
                                               &m_char_handles);
    APP_ERROR_CHECK(err_code);
}


/**@brief Function for handling the Application's BLE Stack events.
 *
 * @param[in] p_ble_evt   Bluetooth stack event.
 */
static void on_ble_evt(ble_evt_t * p_ble_evt)
{
    uint32_t err_code;

    switch (p_ble_evt->header.evt_id)
    {
        case BLE_GAP_EVT_CONNECTED:
            m_conn_handle = p_ble_evt->evt.gap_evt.conn_handle;
            err_code = bsp_indication_set(BSP_INDICATE_CONNECTED);
            APP_ERROR_CHECK(err_code);
            break;

        case BLE_GAP_EVT_DISCONNECTED:
            m_conn_handle = BLE_CONN_HANDLE_INVALID;
            err_code = bsp_indication_set(BSP_INDICATE_IDLE);
            APP_ERROR_CHECK(err_code);

            err_code = bsp_indication_set(BSP_INDICATE_ALERT_OFF);
            APP_ERROR_CHECK(err_code);

            advertising_start();				
            break;

        case BLE_GATTS_EVT_WRITE:
            if ((p_ble_evt->evt.gatts_evt.params.write.handle == m_char_handles.cccd_handle) &&
                (p_ble_evt->evt.gatts_evt.params.write.len == 2))
            {
                err_code = bsp_indication_set(BSP_INDICATE_RCV_OK);
                APP_ERROR_CHECK(err_code);
            }
            break;

        default:
            // No implementation needed.
            break;
    }
}


/**@brief Function for handling the Application's system events.
 *
 * @param[in]   sys_evt   system event.
 */
static void on_sys_evt(uint32_t sys_evt)
{
    switch (sys_evt)
    {
        case NRF_EVT_FLASH_OPERATION_SUCCESS:
        case NRF_EVT_FLASH_OPERATION_ERROR:
            if (m_memory_access_in_progress)
            {
                m_memory_access_in_progress = false;
                advertising_start();
            }
            break;
        default:
            // No implementation needed.
            break;
    }
}


/**@brief Function for dispatching a BLE stack event to all modules with a BLE stack event handler.
 *
 * @details This function is called from the scheduler in the main loop after a BLE stack
 *          event has been received.
 *
 * @param[in] p_ble_evt   Bluetooth stack event.
 */
static void ble_evt_dispatch(ble_evt_t * p_ble_evt)
{
    dm_ble_evt_handler(p_ble_evt);
    on_ble_evt(p_ble_evt);
}


/**@brief Function for dispatching a system event to interested modules.
 *
 * @details This function is called from the System event interrupt handler after a system
 *          event has been received.
 *
 * @param[in]   sys_evt   System stack event.
 */
static void sys_evt_dispatch(uint32_t sys_evt)
{
    pstorage_sys_event_handler(sys_evt);
    on_sys_evt(sys_evt);
}


/**@brief Function for initializing the BLE stack.
 *
 * @details Initializes the SoftDevice and the BLE event interrupt.
 */
static void ble_stack_init(void)
{
    uint32_t err_code;

    // Initialize the SoftDevice handler module.
    SOFTDEVICE_HANDLER_INIT(NRF_CLOCK_LFCLKSRC_XTAL_20_PPM, NULL);

#ifdef S110
    // Enable BLE stack. 
    ble_enable_params_t ble_enable_params;
    memset(&ble_enable_params, 0, sizeof(ble_enable_params));
    ble_enable_params.gatts_enable_params.service_changed = IS_SRVC_CHANGED_CHARACT_PRESENT;
    err_code = sd_ble_enable(&ble_enable_params);
    APP_ERROR_CHECK(err_code);
#endif

    // Register with the SoftDevice handler module for BLE events.
    err_code = softdevice_ble_evt_handler_set(ble_evt_dispatch);
    APP_ERROR_CHECK(err_code);
    // Register with the SoftDevice handler module for BLE events.
    err_code = softdevice_sys_evt_handler_set(sys_evt_dispatch);
    APP_ERROR_CHECK(err_code);

}


/**@brief Function for handling button presses.
 *
 * @param[in] button_action  event generated by button press.
 */
static void button_event_handler(bsp_event_t button_action)
{
    uint32_t err_code;

    if(button_action == BSP_EVENT_KEY_0)
    {
        if (m_conn_handle != BLE_CONN_HANDLE_INVALID)
        {
            static uint8_t value = 0x00;

            ble_gatts_hvx_params_t hvx_params;
            uint16_t               len = sizeof(uint8_t);

            value = (value == 0) ? 1 : 0;

            memset(&hvx_params, 0, sizeof(hvx_params));

            hvx_params.handle = m_char_handles.value_handle;
            hvx_params.type   = BLE_GATT_HVX_NOTIFICATION;
            hvx_params.offset = 0;
            hvx_params.p_len  = &len;
            hvx_params.p_data = &value;

            err_code = sd_ble_gatts_hvx(m_conn_handle, &hvx_params);
            APP_ERROR_CHECK(err_code);

            if (value == 0)
            {
                err_code = bsp_indication_set(BSP_INDICATE_ALERT_OFF);
                APP_ERROR_CHECK(err_code);
            }
            else
            {
                err_code = bsp_indication_set(BSP_INDICATE_ALERT_3);
                APP_ERROR_CHECK(err_code);
            }
        }
    }
}


/**@brief Function for handling the Bond Manager events.
 *
 * @param[in]   p_evt   Data associated to the bond manager event.
 */
static uint32_t device_manager_evt_handler(dm_handle_t const * p_handle,
                                           dm_event_t const  * p_event,
                                           ret_code_t        event_result)
{   uint32_t err_code;

    switch(p_event->event_id)
    {
        case DM_EVT_CONNECTION:
            m_conn_handle = p_event->event_param.p_gap_param->conn_handle;
            err_code = bsp_indication_set(BSP_INDICATE_CONNECTED);
            APP_ERROR_CHECK(err_code);
            break;

        case DM_EVT_DISCONNECTION:
            m_conn_handle = BLE_CONN_HANDLE_INVALID;
            advertising_start();
            break;
    }

    return NRF_SUCCESS;
}


/**@brief Function for the Device Manager initialization.
 */
static void device_manager_init(void)
{
    uint32_t               err_code;
    dm_init_param_t        init_data;
    dm_application_param_t register_param;

    // Initialize persistent storage module.
    err_code = pstorage_init();
    APP_ERROR_CHECK(err_code);

    // Clear all bonded centrals if the Bonds Delete button is pushed.
    err_code = bsp_button_is_pressed(BOND_DELETE_ALL_BUTTON_ID,&(init_data.clear_persistent_data));
    APP_ERROR_CHECK(err_code);

    err_code = dm_init(&init_data);
    APP_ERROR_CHECK(err_code);

    register_param.sec_param.bond         = SEC_PARAM_BOND;
    register_param.sec_param.mitm         = SEC_PARAM_MITM;
    register_param.sec_param.io_caps      = SEC_PARAM_IO_CAPABILITIES;
    register_param.sec_param.oob          = SEC_PARAM_OOB;
    register_param.sec_param.min_key_size = SEC_PARAM_MIN_KEY_SIZE;
    register_param.sec_param.max_key_size = SEC_PARAM_MAX_KEY_SIZE;
    register_param.evt_handler            = device_manager_evt_handler;
    register_param.service_type           = DM_PROTOCOL_CNTXT_GATT_SRVR_ID;

    err_code = dm_register(&m_app_handle, &register_param);
    APP_ERROR_CHECK(err_code);
}


/**@brief Function for the Power manager.
 */
static void power_manage(void)
{
    uint32_t err_code = sd_app_evt_wait();
    APP_ERROR_CHECK(err_code);
}


/**@brief Function for application main entry.
 */

//a UART

#define UART_TX_BUF_SIZE                256
#define UART_RX_BUF_SIZE                256   

void uart_event_handle(app_uart_evt_t * p_event)
{
}


static void uart_init(void)
{
    uint32_t err_code;
    const app_uart_comm_params_t comm_params =
      {
          RX_PIN_NUMBER,
          TX_PIN_NUMBER,
          RTS_PIN_NUMBER,
          CTS_PIN_NUMBER,
          APP_UART_FLOW_CONTROL_DISABLED,
          false,
          UART_BAUDRATE_BAUDRATE_Baud38400
      };

    APP_UART_FIFO_INIT( &comm_params,
                        UART_RX_BUF_SIZE,
                        UART_TX_BUF_SIZE,
												uart_event_handle,
                        APP_IRQ_PRIORITY_LOW,
                        err_code);
    APP_ERROR_CHECK(err_code);
}

//b

//a Timer

static void test_timer_handler(void * p_context)
{
    UNUSED_PARAMETER(p_context);
		printf("Hallo\n");
//	printf("%d\n",hole_temperatur());
}
static app_timer_id_t m_test_timer_id;
void test_timer_create_and_start(void) {
app_timer_create(&m_test_timer_id, APP_TIMER_MODE_REPEATED, test_timer_handler);
app_timer_start(m_test_timer_id, APP_TIMER_TICKS(4000,0), NULL);	
}
//b


int main(void)
{   uint32_t err_code;

    ble_stack_init();
    APP_TIMER_INIT(APP_TIMER_PRESCALER, APP_TIMER_MAX_TIMERS, APP_TIMER_OP_QUEUE_SIZE, NULL);
    APP_GPIOTE_INIT(APP_GPIOTE_MAX_USERS);
    err_code = bsp_init(BSP_INIT_LED | BSP_INIT_BUTTONS, APP_TIMER_TICKS(100, APP_TIMER_PRESCALER), button_event_handler);
    APP_ERROR_CHECK(err_code);
    device_manager_init();
    gap_params_init();
    advertising_init();
    services_init();
    err_code = bsp_event_to_button_assign(SEND_NOTIFICATION_BUTTON_ID, BSP_EVENT_KEY_0);
    APP_ERROR_CHECK(err_code);

    err_code = bsp_buttons_enable(1<<SEND_NOTIFICATION_BUTTON_ID);
    APP_ERROR_CHECK(err_code);

	//a 	
		
		//sd_ble_gap_tx_power_set(4);
		uart_init();
		nrf_delay_ms(1);
		//printf("%s","Test");
		//printf("%3.2fV\n",(messe_spannung()*0.0140625));
	
		err_code = spi_master_init();
		spi_master_evt_handler_reg(SPI_MASTER_0, spi_master_event_handler);
		APP_ERROR_CHECK(err_code);	
		nrf_delay_ms(1);
		
		sensor_reset(); // einmal bei Start #1
		hole_koeffizienten(); // einmal bei Start #2
		
	//a Timer
		//test_timer_create_and_start();	
	//b
		
		//printf("CRC: %s\n",crc_check() ? "True" : "False"); // einmal bei Start #3
		
		messwerte_dezimal_per_uart(0);
		


	/*	uint8_t temp_temp[3] = {0};
		hole_temperatur();
		uint32_t temp_value = hole_druck();
		//MSB also in der Reihenfolge wie es erscheint zu einer 0x123456 zusammensetzen
		temp_temp[2] = (temp_value & 0x000000ff);
		temp_temp[1] = (temp_value & 0x0000ff00) >> 8;
		temp_temp[0] = (temp_value & 0x00ff0000) >> 16;
		
		ble_gatts_value_t test_value;
		test_value.len = sizeof(uint8_t)*3;
		test_value.offset = 0;
		test_value.p_value = (uint8_t*)&temp_temp;		
		sd_ble_gatts_value_set(m_conn_handle,m_char_handles.value_handle, &test_value);
		printf("%d",temp_value); */
		
		
	//b
					
    advertising_start();
		
		
		


    for (;;)
    {
      power_manage();		
//a		

			//messreihe(4000);
			//messwerte_dezimal_per_uart(2000);
/*			
		uint32_t temp_value = hole_druck();
		temp_temp[2] = (temp_value & 0x000000ff);
		temp_temp[1] = (temp_value & 0x0000ff00) >> 8;
		temp_temp[0] = (temp_value & 0x00ff0000) >> 16;
		test_value.p_value = (uint8_t*)&temp_temp;		
		sd_ble_gatts_value_set(m_conn_handle,m_char_handles.value_handle, &test_value);	
*/
//b
    }
}
